#include "as32m0_ssd1306.h"

void init_I2C_SSD1306(uint8_t taddr, I2C_LOC i2c_loc){
	PIN_INFO pin_info;
	switch (i2c_loc){
		case (I2C_PB45):
			pin_info.pin_func = GIO_FUNC1;
			pin_info.pin_stat = GIO_PU;
			PinCtrl_GIOSet(PIN_CTL_GPIOB, GPIO_Pin_4|GPIO_Pin_5, &pin_info);	
			break;
		case (I2C_PC01):
			pin_info.pin_func = GIO_FUNC3;
			pin_info.pin_stat = GIO_PU;
			PinCtrl_GIOSet(PIN_CTL_GPIOC, GPIO_Pin_0|GPIO_Pin_1, &pin_info);	
			break;		
		case (I2C_PC67):
			pin_info.pin_func = GIO_FUNC1;
			pin_info.pin_stat = GIO_PU;
			PinCtrl_GIOSet(PIN_CTL_GPIOC, GPIO_Pin_6|GPIO_Pin_7, &pin_info);	
			break;
		case (I2C_PD23):
			pin_info.pin_func = GIO_FUNC3;
			pin_info.pin_stat = GIO_PU;
			PinCtrl_GIOSet(PIN_CTL_GPIOD, GPIO_Pin_2|GPIO_Pin_3, &pin_info);	
			break;		
	}
	I2C_MasterConfig(APB_I2C,taddr,1,1,1,8,8,0); // Slave address : 0x3D-128x64 pixels
}

void I2C_SSD1306_command(uint8_t command){
	uint8_t data[2];
	data[0] = 0x00;
	data[1] = command;
	I2CWirteBytes(APB_I2C, data, 2);
}

void I2C_SSD1306_data(uint8_t value){
	uint8_t data[2];
	data[0] = 0x40;
	data[1] = value;
	I2CWirteBytes(APB_I2C, data, 2);
}

void I2C_SSD1306_display_on(void){
	I2C_SSD1306_command(SSD1306_DISPLAYON);
}
void I2C_SSD1306_display_off(void){
	I2C_SSD1306_command(SSD1306_DISPLAYOFF);
}

void I2C_SSD1306_chargepump_on(void){
	I2C_SSD1306_command(SSD1306_CHARGEPUMP);
	I2C_SSD1306_command(0x14);
}

void I2C_SSD1306_chargepump_off(void){
	I2C_SSD1306_command(SSD1306_CHARGEPUMP);
	I2C_SSD1306_command(0x10);
}

void I2C_SSD1306_set_precharge_period(uint8_t value){
	I2C_SSD1306_command(SSD1306_SETPRECHARGE);
	I2C_SSD1306_command(value);	
}

void I2C_SSD1306_set_contrast(uint8_t value){
	I2C_SSD1306_command(SSD1306_SETCONTRAST);
	I2C_SSD1306_command(value);		
}

void I2C_SSD1306_setAddrmode(uint8_t value){
	I2C_SSD1306_command(SSD1306_MEMORYMODE);
	I2C_SSD1306_command(value & 0x03);			
}

void I2C_SSD1306_setPageaddr(uint8_t start, uint8_t end){
	I2C_SSD1306_command(SSD1306_MEMORYMODE);
	I2C_SSD1306_command(start & 0x07);			
	I2C_SSD1306_command(end & 0x07);	
}

void I2C_SSD1306_display_clear(void){
	uint8_t i, j;
	I2C_SSD1306_setPageaddr(0, 7);
	for(i=0;i<8;i++)
		for(j=0;j<128;j++)
			I2C_SSD1306_data(0x00);
	I2C_SSD1306_setPageaddr(0, 7);
}

void I2C_SSD1306_initOLED(void){
	I2C_SSD1306_display_off();
	I2C_SSD1306_chargepump_on();
	I2C_SSD1306_set_precharge_period(0x22);
	I2C_SSD1306_set_contrast(180);
	I2C_SSD1306_command(SSD1306_SEGREMAP);
	I2C_SSD1306_command(SSD1306_COMSCANINC);
	I2C_SSD1306_setAddrmode(0x00);
	I2C_SSD1306_display_clear();
	__NOP;
	I2C_SSD1306_display_on();
}

void I2C_SSD1306_drawBitMap(uint8_t Bitmap []){
	uint8_t i, j;
	I2C_SSD1306_display_off();
	I2C_SSD1306_setPageaddr(0, 7);
	for(i=0;i<8;i++)
		for(j=0;j<128;j++)
			I2C_SSD1306_data(Bitmap[j+128*(i+1)]);
	I2C_SSD1306_setPageaddr(0, 7);
	I2C_SSD1306_display_on();
}

void I2C_SSD1306_LeftHScrol(uint8_t start_page, uint8_t frame, uint8_t end_page){
	I2C_SSD1306_command(SSD1306_LEFT_HORIZONTAL_SCROLL);
	I2C_SSD1306_command(0x00);
	I2C_SSD1306_command(start_page & 0x7);
	I2C_SSD1306_command(frame & 0x7);
	I2C_SSD1306_command(end_page & 0x7);
	I2C_SSD1306_command(0x00);
	I2C_SSD1306_command(0xFF);
	I2C_SSD1306_command(SSD1306_ACTIVATE_SCROLL);
}

void I2C_SSD1306_RightHScrol(uint8_t start_page, uint8_t frame, uint8_t end_page){
	I2C_SSD1306_command(SSD1306_RIGHT_HORIZONTAL_SCROLL);
	I2C_SSD1306_command(0x00);
	I2C_SSD1306_command(start_page & 0x7);
	I2C_SSD1306_command(frame & 0x7);
	I2C_SSD1306_command(end_page & 0x7);
	I2C_SSD1306_command(0x00);
	I2C_SSD1306_command(0xFF);
	I2C_SSD1306_command(SSD1306_ACTIVATE_SCROLL);
}

void I2C_SSD1306_StopScrol(void){
	I2C_SSD1306_command(SSD1306_DEACTIVATE_SCROLL);
}
